/*
Daniel Grados Rico, TFG, Àrea Arduino
Tanc de revelatge intel·ligent per a la fotografia analògica
Grau de Multimèdia, Curs 2022/2023
*/

//S'afegeixen les llibreries per a la pantalla
#include <URTouch.h>
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>

//S'afegeixen la llibreria per al servomotor
#include <Servo.h>

//S'afegeixen llibreries per al sensor de temperatura
#include <OneWire.h>                
#include <DallasTemperature.h>
 
OneWire ourWire(8); //S'estableix el pin 8 per al sensor de temperatura
DallasTemperature sensor(&ourWire); 

 
//Pins digitals per a la pantalla tàctil
#define TFT_DC 9
#define TFT_CS 10
#define t_SCK  3
#define t_CS   4
#define t_MOSI 5
#define t_MISO 6
#define t_IRQ  7

//Variable del sensor de temperatura
float temper;
 
//Variables per al temporitzador
long minuts;
long minutsPantalla;
long segons;
long segonsPantalla;

//Les següents variables són de tipus unsigned long perquè hauran d'operar amb un nombre major
unsigned long tempsActual;
unsigned long tempsPassat;
unsigned long inici = 0;
unsigned long duracio = 0;
unsigned long duracioServo;

//Variables booleanes 
bool tempo_ok = false;
bool tempo_normal = true;
bool tempo_instant = false;

//Variables necessaries per a les pauses del temporitzador i les agitacions
unsigned long valorPausa;
bool pRotacioOk = false;
bool pAturadaOk = false;
bool sRotacioOk = false;
bool sAturadaOk = false;

//Variables per al servomotor
Servo myservo;
int posicioServo = myservo.read();
bool accioServo = false;
bool intervalServo = false;
unsigned long tempsServo;
bool pausa = false;

//Variables de coordenades per al reconeixement de la pantalla tàctil
unsigned long x, y;

//Variables per els temps de les agitacions. Aquestes variables es podran modificar per la persona més endavant
unsigned long primeraRotacio;
unsigned long primeraAturada;
unsigned long segonaRotacio;
unsigned long segonaAturada;

//Variable booleana per al botó "!" i activar el mode manual
bool instantOk = false;

//Instància la pantalla 
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);
 
//Instància de l'apartat tàctil de la pantalla
URTouch ts(t_SCK, t_CS, t_MOSI, t_MISO, t_IRQ);
 
void setup(){

  Serial.begin(9600);
  sensor.begin();   //S'inicia el sensor
  myservo.attach(2); //S'estableix el pin 2 per al servomotor

  //Configuració de la pantalla TFT
  tft.begin(); //S'inicia tft
  tft.setRotation(3); //Posició horitzontal de la pantalla
  tft.fillScreen(ILI9341_BLACK); //Fons negre

  //Missatge principal
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(2);
  tft.setCursor(50, 100);
  tft.print("Daniel Grados Rico");
  tft.setCursor(27, 130);
  tft.print("Treball Final de Grau");
  tft.setCursor(25, 180);
  tft.print("Grau de Multimedia");
  tft.setCursor(25, 210);
  tft.print("Curs 2022/2023");
  delay (7000);//Es deixa un temps per llegir el missatge principal

  //S'esborra el missatge anterior
  tft.fillScreen(ILI9341_BLACK); //Fons negre per esborrar el text anterior
  tft.setTextSize(5);//S'estableix la mida del text en 5 per als pròxims elements

  //S'executa el void dels elements a la pantalla (botons i altres figures) per tal de veure'ls
  elementsPantalla ();

  //Configuració de la pantalla tàctil
  ts.InitTouch(); //S'inicia la pantalla tàctil
  ts.setPrecision(PREC_MEDIUM); //Precissió de la pantalla tàctil
  
  //S'imprimeix el temps en pantalla
  tft.setTextColor(ILI9341_WHITE); //S'estableix el color blanc per al text
  tft.setTextSize(5); //Mida del text
  tft.setCursor(172, 45); //Es posiciona en coordenades on aniran els minuts
  tft.print(String(minuts)); //S'imprimeix la variable minuts en la pantalla
  tft.setCursor(220, 45); //Es posiciona en coordenades on anirà : del temporitzador
  tft.print(":"); //S'imprimeix : en pantalla
  tft.setCursor(270, 45); //S'estableix en coordenades on aniran els segons
  tft.print(segons); //S'imprimeix en pantalla els segons 

//Iniciem el void del sensor de temperatura
sensorTemperatura ();

} //Final del void setup
 

void loop(){

  tempsActual = millis(); //Donem valor en mil·lisegons a la variable tempsActual
  
  //Condició per a que detecció tàctil
  while(ts.dataAvailable()){ //Mentre que la pantalla tàctil sigui polsada
    ts.read(); //Es llegeix el valor
    x = ts.getX(); //X agafa el valor en la coordenada X
    y = ts.getY(); //Y agafa el valor en la coordenada Y

//Els següents voids s'activen mitjançant la pantalla tàctil
  tempsPantalla (); //Botons per canviar el minutatge del temporitzador
  agitacions (); //Botons per a les agitacions M, I, C, ! i F
  sensorTemperatura (); //Botó per a la lectura del sensor de temperatura
  }//Final de la condició while

//S'inicia el void del temporitzador de les agitacions del servomotor
  temporitzadorMotor ();

//Condició per al mode "!", si s'activa, el servomotor s'activara fins que sigui false
if (instantOk == true){
     motorInstant ();
}

/*Els diferents tipus de mides de lletres i colors a la pantalla obliguen a "reiniciar"
els valors dels elements a la pantalla per tal de no veure variacions als elements per si hereten una configuració
que no és la seva, per exemple, la mida de la lletra i color dels números del temporitzador
es diferencia de la lectura del sensor de temperatura.*/

//Reiniciem el color i la mida del text per a què no es canviïn en la interfície
tft.setTextColor(ILI9341_WHITE);
tft.setTextSize(5);

//Condició per aturar i congelar els temps de les agitacions i aturades quan es polsa cancel·lar "X"
if (pausa == true){
  
  valorPausa = tempsPassat; //valorPausa guarda el valor del tempsPassat
  
  //Si es polsa cancel·lar en la primera rotació
  if (pRotacioOk == true){
  primeraRotacio = primeraRotacio - valorPausa; //Les rotacions hereten el valor amb el valor que tenien abans de la pausa
  primeraAturada = primeraAturada - valorPausa; //La primera aturada també s'ha de restar ja que el seu valor és la suma del valor de la variable primeraAturada + primeraRotacio
  } 

//Si es polsa cancel·lar en la primera aturada
  if (pAturadaOk == true){
  primeraAturada = primeraAturada - valorPausa; //Les rotacions hereten el valor amb el valor que tenien en la pausa
  }   

//Si es polsa cancel·lar en la segona rotació
  if (sRotacioOk == true){
  segonaRotacio = segonaRotacio - valorPausa; //Les rotacions hereten el valor amb el valor que tenien en la pausa
  } 
//Si es polsa cancel·lar en la segona aturada
  if (sAturadaOk == true){
  segonaAturada = segonaAturada - valorPausa; //Les rotacions hereten el valor amb el valor que tenien en la pausa
  } 

  //Canvia l'estat del boolea i surt del bucle per tal de només entrar una vegada 
  pausa = false;
}

}//Final del loop


 
//Void per a mostrar i canviar el temps a la pantalla
void tempsPantalla (){

 //Reconeixement tàctil "+" dels minuts
   if (x >= 155 && x <= 218 && y >= 7 && y <= 39){ //Coordenades del botó + dels minuts
    minuts ++; //Sumem 1 als minuts
    tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); // Esborra el valor anterior
    delay (500);//Donem un espai de temps per actualitzar-se
   }//Final del reconeixement tàctil + minuts

  //Reconeixement tàctil "-" dels minuts
   if (x >= 155 && x <= 218 && y >= 85 && y <= 117){ //Coordenades del botó - dels minuts
   minuts --; //Restem 1 als minuts
   tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); // Esborra el valor anterior
   delay (500); //Donem un espai de temps per actualitzar-se
  
  //Condició per evitar que hi hagi nombres negatius
  if (minuts < 0){
    minuts = 0;
  } 
}//Final del reconeixement tàctil - minuts

  //Reconeixement "+" dels segons
   if (x >= 250 && x <= 313 && y >= 7 && y <= 39){ //Coordenades del botó + dels segons
     segons += 15; //Sumem 15 segons 
    tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); // Esborra el valor anterior
    delay (500); //Donem un espai de temps per actualitzar-se
  
  //Condició per evitar que hi hagi nombres més grans que 45 segons
    if (segons > 45){
      segons = 45;
    }
  } //Final del reconeixement tàctil + segons

  //Reconeixement "-" dels segons
   if (x >= 250 && x <= 313 && y >= 85 && y <= 117){ //Coordenades del botó - dels segons
     segons -= 15; //Restem 15 segons
    tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); // Esborra el valor anterior
    delay (500); //Donem un espai de temps per actualitzar-se
  
  //Condició per evitar que hi hagi nombres negatius
  if (segons < 0){
    segons = 0;
  }
} //Final del reconeixement tàctil - segons

//Si els minuts és un número d'una xifra es desplaçarà cap a la dreta
   if (minuts < 10){
     tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); //Esborra el valor anterior
     tft.setCursor(172, 45); //Es posiciona el nou valor
     tft.print(String(minuts)); //S'imprimeix en pantalla la variable minuts
   }
   else{ //Si el minut és de dues xifres, es desplaçarà 20px cap a l'esquerra
     tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); 
     tft.setCursor(152, 45);
     tft.print(String(minuts));
   }

//Si els segons és un número d'una xifra es desplaçarà cap a la dreta
   if (segons < 10){
     tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); //Esborra el valor anterior
     tft.setCursor(270, 45); //Es posiciona el nou valor
     tft.print(String(segons)); //S'imprimeix en pantalla la variable minuts
   }
   else{ //Si el segon és de dues xifres, es desplaçarà 20px cap a l'esquerra
     tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); 
        tft.setCursor(250, 45);
        tft.print(String(segons));
   }

  //Botons "START" i "X" per posar en marxa o cancelar el temporitzador
    if (x >= 165 && x <= 315 && y >= 190 && y <= 230){  //Reconeixement tàctil de la posició
      duracio = minuts * 60000 + segons * 1000; //Pasem els minuts i els segons a mil·lisegons
      duracioServo = duracio; //Hereta el valor de duracio
      inici = tempsActual; //Hereta el valor de tempsActual
      minutsPantalla = minuts;  //Hereta el valor de minuts  
      tempo_ok = true;  //Es canvia l'estat 
      accioServo = true; //Es canvia l'estat
    }//Final de la condició d'START i X


//Reconeixement del botó cancelar
     if (x >= 5 && x <= 155 && y >= 190 && y <= 230){ //Reconeixement de les coordenades
      tempo_ok = false; //Es canvia l'estat de la variable 
      tempsActual = 0; //Es reinicia el valor de la variable tempsActual
      accioServo = false; //Es canvia l'estat de la variable
      pausa = true; //S'activa la variable pausa per tal que els valors actuals quedin guardats. Es pot veure a la condició situada al loop
      }//Final condició reconeixements cancel·lar

} //Final void tempsPantalla

//Void per a la creació dels elements gràfics a la pantalla
void elementsPantalla(){

  //Creació requadre "+" per sumar minuts
  tft.fillRoundRect (155, 7, 63, 32, 5, 0xC618);
  tft.setTextColor(ILI9341_BLUE);
  tft.setTextSize(4);
  tft.setCursor(175, 8);
  tft.print("+");

  //Creació requadre "-" per restar minuts
  tft.fillRoundRect (155, 85, 63, 32, 5, 0xC618);
  tft.setTextColor(ILI9341_BLUE);
  tft.setTextSize(4);
  tft.setCursor(176, 87);
  tft.print("-");

  //Creació requadre "+" per sumar segons
  tft.fillRoundRect (250, 7, 63, 32, 5, 0xC618);
  tft.setTextColor(ILI9341_BLUE);
  tft.setTextSize(4);
  tft.setCursor(270, 8);
  tft.print("+");

  //Creació requadre "-" per restar segons
  tft.fillRoundRect (250, 85, 63, 32, 5, 0xC618);
  tft.setTextColor(ILI9341_BLUE);
  tft.setTextSize(4);
  tft.setCursor(271, 87);
  tft.print ("-");

   //Requadre acceptar "START"
  tft.fillRoundRect (165, 190, 150, 40, 5, ILI9341_GREEN);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(3);
  tft.setCursor(195, 200);
  tft.print("START");

    //Requadre cancel·lar "X"
  tft.fillRoundRect (5, 190, 150, 40, 5, ILI9341_RED);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(3);
  tft.setCursor(70, 200);
  tft.print("X");

/*La següent funció és per establir els colors dels botons de manera automàtica i estalviar linies.
Dins aquesta funció es pot trobar els disseny predeterminats dels botons*/
//Botons per a les agitacions
colorBotons();

//Reiniciem la mida del text
 tft.setTextSize(5);

} //Final void elementsPantalla


/*La següent funció està destinada als temps d'agitacions i aturades per al revelatge.
La persona podrà modificar les agitacions i aturades al seu gust en cada un del tipus.
S'ha intentat oferir de manera fàcil aquesta tasca només canviant un valor de les següents
variables: primeraRotació, primeraAturada, segonaRotacio i segonaAturada.*/
void agitacions (){
  
  //Si es polsa "F" (Fixador):
  if (x >= 5 && x <= 65 && y >= 5 && y <= 85){ //Reconeixment tàctil de les coordenades

  colorBotons();//S'activa el void per posar del mateix colors els altres botons
  
  //Es canvia el color del botó del fixador
  tft.fillRoundRect (5, 5, 60, 80, 5,ILI9341_WHITE);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(4);
  tft.setCursor(25, 30);
  tft.print("F");

//Reiniciem la mida del text 
  tft.setTextSize(5);

/*
//Copia de seguretat 
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 0; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 0; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 0; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 0;  //Substituir el valor per el temps desitjat per a la segona aturada en segons
  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
*/

//Part modificable
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 30; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 30; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 30; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 30;  //Substituir el valor per el temps desitjat per a la segona aturada en segons

//Part NO modificable  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
} //Final del botó "F" fixador


//Si es selecciona l'agitació Mitjana "M":
  if (x >= 80 && x <= 140 && y >= 5 && y <= 85){ //Reconeixement tàctil de les coordenades

  colorBotons();//S'activa el void per posar del mateix colors els altres botons
 
  //Canvia el color del botó M
  tft.fillRoundRect (80, 5, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(5);
  tft.setCursor(97, 28);
  tft.print("M");
  
/*
//Copia de seguretat 
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 0; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 0; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 0; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 0;  //Substituir el valor per el temps desitjat per a la segona aturada en segons
  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
*/

//Part modificable
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 60; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 180; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 10; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 180;  //Substituir el valor per el temps desitjat per a la segona aturada en segons

//Part NO modificable  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
}//Final del botó "M" mínima

//Si es polsa "I":
  if (x >= 5 && x <= 65 && y >= 100 && y <= 180){ //Reconeixement tàctil de les coordenades
 
  colorBotons ();//S'activa el void per posar del mateix colors els altres botons
  
  //Canvia el color del botó "I"
  tft.fillRoundRect (5, 100, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(5);
  tft.setCursor(22, 122);
  tft.print("I");

/*
//Copia de seguretat 
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 0; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 0; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 0; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 0;  //Substituir el valor per el temps desitjat per a la segona aturada en segons
  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
*/

//Part modificable
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 60; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 60; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 30; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 60;  //Substituir el valor per el temps desitjat per a la segona aturada en segons

//Part NO modificable  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
} //Final del botó "I" Intermitent 

  //Si es polsa "C":
  if (x >= 80 && x <= 140 && y >= 100 && y <= 180){ //Reconeixement tàctil de les coordenades 
  
  colorBotons ();//S'activa el void per posar del mateix colors els altres botons
  
  //Es canvia el color del botó "C"
  tft.fillRoundRect (80, 100, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(5);
  tft.setCursor(97, 122);
  tft.print("C");
 
/*
//Copia de seguretat 
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 0; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 0; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 0; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 0;  //Substituir el valor per el temps desitjat per a la segona aturada en segons
  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
*/

//Part modificable
//Valors en mil·lisegons de les rotacions i aturades per a l'agitació personalitzada
primeraRotacio = 60; //Substituir el valor per el temps desitjat del primer moviment en segons (minuts x 60) 
primeraAturada = 0; //Substituir el valor per el temps desitjat de la primera aturada en segons (minuts x 60)  
segonaRotacio = 60; //Substituir el valor per el temps desitjat del segon moviment en segons 
segonaAturada = 0;  //Substituir el valor per el temps desitjat per a la segona aturada en segons

//Part NO modificable  
//No tocar
primeraRotacio = primeraRotacio * 1000; //No tocar
primeraAturada = primeraRotacio + (primeraAturada * 1000); //No tocar
segonaRotacio = segonaRotacio * 1000; //No tocar
segonaAturada = segonaAturada * 1000; //No tocar
} //Final del botó "C" Contínua
  
  //Si es polsa "!":
  if (x >= 155 && x <= 230 && y >= 130 && y <= 180){
  
  colorBotons();//S'activa el void per posar del mateix colors els altres botons
  
  instantOk = !instantOk; //Canviem l'estat booleà cada vegada que es pulsa el botó "!"
  
  //Es canvia el color del botó "!"
  if (instantOk == true){ //Si el boolea anterior és true 
  tft.fillRoundRect (155, 130, 75, 50, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_BLACK);
  tft.setTextSize(4);
  tft.setCursor(185, 140);
  tft.print("!");

  //Reiniciem mida text
  tft.setTextSize(5);
  }
   else {  //Si es false
  //Botó agitació instantanea desactivada
  tft.fillRoundRect (155, 130, 75, 50, 5, ILI9341_BLACK); //Pintem de negre per resetejar el color
  tft.drawRoundRect (155, 130, 75, 50, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(4);
  tft.setCursor(185, 140);
  tft.print("!");

  //Reiniciem mida text
  tft.setTextSize(5);
  }

  } //Final del botó "!" Manual
  

}//Final del void agitacions

/* Per evitar escriure línies repetides com poden ser les creacions dels botons d'agitacions
cada vegada que es pulsa i canvia un color, es va crear una funció per tal d'executar-la
cada vegada que s'havia de reiniciar els dissenys creats com els botons de les agitacions.
Executant aquest void, es dibuixarà automàticament a la pantalla els elements gràfics. */
void colorBotons (){
  
  //Botó agitació manual
  tft.drawRoundRect (155, 130, 75, 50, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(4);
  tft.setCursor(185, 140);
  tft.print("!");
  
  //Botó fixador
  tft.fillRoundRect (5, 5, 60, 80, 5,ILI9341_BLACK);
  tft.drawRoundRect (5, 5, 60, 80, 5,ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(4);
  tft.setCursor(25, 30);
  tft.print("F");
  //Reiniciem mida text
  tft.setTextSize(5);

  //Botó agitació Miníma
  tft.fillRoundRect (80, 5, 60, 80, 5, ILI9341_BLACK);
  tft.drawRoundRect (80, 5, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(5);
  tft.setCursor(97, 28);
  tft.print("M");

  //Botó agitació Intermitent
  tft.fillRoundRect (5, 100, 60, 80, 5, ILI9341_BLACK);
  tft.drawRoundRect (5, 100, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(5);
  tft.setCursor(22, 122);
  tft.print("I");

  //Botó agitació Contínua
  tft.fillRoundRect (80, 100, 60, 80, 5, ILI9341_BLACK);
  tft.drawRoundRect (80, 100, 60, 80, 5, ILI9341_WHITE);
  tft.setTextColor(ILI9341_WHITE);
  tft.setTextSize(5);
  tft.setCursor(97, 122);
  tft.print("C");

} //Final void colorBotons

//Funció per activar el servomotor quan es polsa el botó manual "!"
void motorInstant (){
      posicioServo += 1;
      if (posicioServo > 180) {
      posicioServo = 0;  // Reiniciar la posición del servo a 0 grados
    }
        myservo.write(posicioServo);
        delay (5);
}//Final void motorInstant

//El següent void està destinat al pas del temps del temporitzador i atribuir les agitacions al servomotor
void temporitzadorMotor (){
//Si s'ha polsat el botó verd s'activa el temporitzador
  if (tempo_ok == true){
    minutsPantalla = minuts;
    tempsPassat = tempsActual - inici;
      if (tempsPassat >= duracio){
        tempo_ok = false;
        tempsPassat = duracio;
      }
    segons = (duracio - tempsPassat) / 1000; //La variable segons pren el valor per imprimir-se en pantalla
    minuts = segons / 60; //La variable minuts pren el valor normal (sense mil·lisegons)
    segons = segons % 60; //La variable segons pren el valor normal (sense mil·lisegons)

   //Condició per canviar a la pantalla els minuts
   if (minuts != minutsPantalla){
    tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); // Esborra el valor anterior
    minutsPantalla = minuts;
   }
   //Es reescriuen els valors dels minuts a la pantalla
    delay (1000);
   if (minuts < 10){ //Si els minuts tenen només una xifra es desplaçen a la dreta
     tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); // Elimina el valor anterior
     tft.setCursor(172, 45);
     tft.print(String(minuts));
   }
   else{ //Si els minuts tenen més d'una xifra es desplaçen a l'esquerra
     tft.fillRect(152, 42, 69, 40, ILI9341_BLACK); // Elimina el valor anterior
     tft.setCursor(152, 45);
     tft.print(String(minuts));
   }
  
   //Es reescriuen els valors dels segons a la pantalla
   if (segons < 10){ //Si els segons tenen només una xifra es desplaçen a la dreta
     tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); // Elimina el valor anterior
     tft.setCursor(270, 45);
     tft.print(String(segons)); 
   }
   else{ //Si els segons tenen més d'una xifra es desplaçen a l'esquerra
    tft.fillRect(245, 42, 69, 40, ILI9341_BLACK); // Elimina el valor anterior
    tft.setCursor(250, 45);
    tft.print(String(segons));
   }
  }

  //Primera part d'agitacions
  if (tempo_ok == true && accioServo == true){
      if (primeraRotacio >= tempsPassat){
      posicioServo += 180; //Rotació del servomotor fins als 180 graus
      
      //El següent booleà s'activa per si es polsa "X" o pausa, guardi el valor actual només la variable primeraRotacio. 
      pRotacioOk = true; //
      pAturadaOk = false;
      sRotacioOk = false;
      sAturadaOk = false;
      }

  if (posicioServo > 180) {
      posicioServo = 0; // si la posició supera els 180 grados torna a la posicio inicial
 }
    
    myservo.write(posicioServo); // El servomotor torna a la posició inical
    
    if (primeraRotacio <= tempsPassat){   
    
     //El següent booleà s'activa per si es polsa "X" o pausa, guardi el valor actual només la variable primeraAturada. 
     pRotacioOk = false;
      pAturadaOk = true;
      sRotacioOk = false;
      sAturadaOk = false;
     
      primeraRotacio = 0;
    }

    //El servomotor s'atura per primera vegada
    if (primeraAturada <= tempsPassat){
      tempsServo = tempsPassat; //Hereta el temps passat actualment
      intervalServo = false;
      accioServo = false;
      primeraAturada = 0; //Una vegada acabada la primeraAturada, es dona valor 0 per tal de no reiniciar-se de nou amb l'antic valor
    }
  } 

  //Segona part dels intervals
  if (tempo_ok == true && accioServo == false && intervalServo == false){
    if ((tempsServo + segonaRotacio) >= tempsPassat && intervalServo == false) {
     posicioServo += 180; //El servomotor girarà fins als 180 graus

    //El següent booleà s'activa per si es polsa "X" o pausa, guardi el valor actual només la variable segonaRotacio. 
      pRotacioOk = false;
      pAturadaOk = false;
      sRotacioOk = true;
      sAturadaOk = false;
    }
    else {
      tempsServo = tempsPassat;
      intervalServo = true;      
    }
    myservo.write(posicioServo); //S'escriu el valor del servomotor


//Torna el servomotor a la posició de 0
    if (posicioServo > 180) {
      posicioServo = 0; // si la posició supera els 180 graus, torna a la posició inicial
    } 
    myservo.write(posicioServo);
  }

//Segona aturada
  if (tempo_ok == true && accioServo == false && intervalServo == true){
    if (tempsServo + segonaAturada >= tempsPassat){
     posicioServo = 0; //El servomotor s'atura als 0 graus
     myservo.write(posicioServo);

      //El següent booleà s'activa per si es polsa "X" o pausa, guardi el valor actual només la variable segonaAturada. 
      pRotacioOk = false;
      pAturadaOk = false;
      sRotacioOk = false;
      sAturadaOk = true;
    }
    else {
      myservo.write(posicioServo);
      tempsServo = tempsPassat;
      intervalServo = false; 
    }
  }

//Condició per saber si s'ha pulsat el botó de cancelar
       if (x >= 5 && x <= 155 && y >= 190 && y <= 230){
      tempo_ok = false;
      tempsActual = 0;
      accioServo = false;
      }
} //Final void temporitzadorMotor

//La següent funció està dissenyada per a donar lectures de la temperatura cada vegada que es polsi el botó destinat
void sensorTemperatura () {

//Comprovació de la temperatura i validació per a la possada en marxa
  if (x >= 235 && x <= 315 && y >= 130 && y <= 180)
  
    sensor.requestTemperatures();   //S'envia l'ordre per llegir la temperatura
    temper= sensor.getTempCByIndex(0); //S'obte la temperatura en ºC
    if (temper <= 25 && temper >= 17){ //Si la temperatura està compresa entre aquests graus centígrads, és correcta
    tft.fillRoundRect (235, 130, 80, 50, 5, ILI9341_BLUE);
    tft.setTextColor(ILI9341_WHITE);
      tft.setTextSize(2);
      tft.setCursor(245, 147);
      tft.print(temper);
      tft.setTextSize(5);
      }
    else if (temper >= 25 || temper <= 17){ //Si la temperatura no està compresa entre aquests graus centígrads, no és correcta
      tft.fillRoundRect (235, 130, 80, 50, 5, ILI9341_RED); //Avís visual d'alerta en vermell
      tft.setTextColor(ILI9341_WHITE);
      tft.setTextSize(2);
      tft.setCursor(245, 147);
      tft.print(temper);
      tft.setTextSize(5);
      tft.setTextSize(5);
    }

} //Final void sensorTemperatura

